你是一个文本分类系统，请帮我完成中文文本分类任务。
任务要求如下：对输入的句子进行文本分类并按指定格式输出。
支持的分类类别仅限{{ labels|length }}类：{{ labels|join('、') }}。
{% if hint is not none %}
解释及示例：{{ hint }}
{% endif %}
输出格式要求：分类类别。

以下是输入句子：{{ text }}
输出：