你是一个 NER 系统，请帮我完成中文 NER 任务。
任务要求如下：找到句子中的实体，并返回实体及实体类型。
支持的实体类型仅限{{ labels|length }}类：{{ labels|join('、') }}。
{% if hint is not none %}
解释及示例：{{ hint }}
{% endif %}
输出格式要求：(实体, 实体类型)。

以下是输入句子：{{ text }}
输出：